import os, sys, time, configparser
import serial
from ymodem import YModem
from radio_bridge import HatControl, RaspiHatControl

def isRaspi():
    try:
        with open('/proc/cpuinfo') as f:
            for line in f:
                line = line.strip()
                if line.startswith('Hardware') and 'BCM' in line: #broadcom
                    return True
    except:
        pass

    return False


def readLineUntil(serial_io, msgStart):
    while True:
        logMsg = readLine(serial_io, True)
        if logMsg.startswith(msgStart):
            break


def readLine(serial_io, blocking):
    if blocking:
        line = serial_io.readline().decode("ascii")
        print("[Gateway Hat]: " + line, end='')
        return line

    while serial_io.inWaiting() > 0:
        print("[Gateway Hat]: " + serial_io.readline().decode("ascii"), end='')

if __name__ == '__main__':
    if len(sys.argv) < 2:
        print("Usage: upload.py flashfile.bin [serialDevice] [baudrate]")
        exit(1)

    binaryFile = sys.argv[1]

    serialDevice = None
    serialSpeed = 500000 # CANIQUE devices run USART @ 500 kbps

    if len(sys.argv) > 2:
        serialDevice = sys.argv[2]

    if len(sys.argv) > 3:
        serialSpeed = int(sys.argv[3])


    if not serialDevice:
        parser = configparser.RawConfigParser()
        with open("/etc/cnq-radio-bridge") as stream:
            parser.read_string(stream.read())
            serialDevice = parser['UART'].get('device')



    print("Using serial device {} @ speed {} bps".format(serialDevice, serialSpeed))


    serial_io = serial.Serial(serialDevice, baudrate=serialSpeed, timeout=0.1)
    #serial_io.parity = "N"
    #serial_io.bytesize = 8
    #serial_io.stopbits = 1
    #serial_io.timeout = 2

    def sender_getc(size):
        return serial_io.read(size) or None

    def sender_putc(data, timeout=15):
        return serial_io.write(data)

    ## reset canique gateway hat
    if isRaspi():
        hatControl = RaspiHatControl.HatControl()
    else:
        hatControl = HatControl.HatControl()

    hatControl.reset()

    #read multiple lines: wait for press key prompt
    readLineUntil(serial_io, "Press")

    sender = YModem.YModem(sender_getc, sender_putc, data_pad=b'\xFF')
    os.chdir(sys.path[0])

    print("Sending command 'd' for firmware upload to Canique device...")
    serial_io.write('d'.encode()) # start download - specific command to CANIQUE devices

    #wait for "Transferring..." info
    readLineUntil(serial_io, "Transferring")
    print("")

    file_path = os.path.abspath(binaryFile)
    sent = sender.send_file(file_path)

    print("Sent {} bytes".format(sent))

    readLine(serial_io, False)

    if sent == os.path.getsize(file_path):
        print("Sending command 'r' ...")
        serial_io.write('r'.encode()) # run firmware - specific command to CANIQUE devices

    #wait until we get success msg (img must be decrypted/verified in this time)
    serial_io.timeout = 5.5
    while True:
        logMsg = readLine(serial_io, True)
        if logMsg.startswith("Starting image"):
            break
        if logMsg.startswith("No valid image found"):
            serial_io.close()
            raise SystemExit(3)

    serial_io.close()
